@@ -1,6 +1,4 @@
-/*
- * -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-
- */
+/* -*- mode: C; indent-tabs-mode: t; tab-width: 8; c-basic-offset: 2; -*- */
 
 /*
  * This file is part of Seed, the GObject Introspection<->Javascript bindings.
@@ -16,7 +14,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with Seed.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) Robert Carr 2008 <carrr@rpi.edu>
+ * Copyright (C) Robert Carr 2009 <carrr@rpi.edu>
  */
 
 #ifndef _SEED_BUILTINS_H_
@@ -25,12 +23,14 @@
 #include "seed-private.h"
 
 extern JSValueRef seed_print_ref;
+extern JSValueRef seed_printerr_ref;
 
 void seed_init_builtins (SeedEngine * local_eng, gint * argc, gchar *** argv);
 
 extern JSClassRef seed_argv_class;
-typedef struct _SeedArgvPrivates {
-  gchar ** argv;
+typedef struct _SeedArgvPrivates
+{
+  gchar **argv;
   gint argc;
 } SeedArgvPrivates;
 
