@@ -1,18 +1,20 @@
+/* -*- mode: C; indent-tabs-mode: t; tab-width: 8; c-basic-offset: 2; -*- */
+
 /*
  * This file is part of Seed, the GObject Introspection<->Javascript bindings.
  *
- * Seed is free software: you can redistribute it and/or modify 
+ * Seed is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation, either version 3 of
- * the License, or (at your option) any later version. 
- * Seed is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
- * GNU Lesser General Public License for more details. 
- * You should have received a copy of the GNU Lesser General Public License 
+ * the License, or (at your option) any later version.
+ * Seed is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ * You should have received a copy of the GNU Lesser General Public License
  * along with Seed.  If not, see <http://www.gnu.org/licenses/>.
- 
- * Copyright (C) Robert Carr 2008 <carrr@rpi.edu>
+ *
+ * Copyright (C) Robert Carr 2009 <carrr@rpi.edu>
  */
 
 #include "seed-private.h"
@@ -34,6 +36,8 @@ seed_closure_finalize (JSObjectRef object)
   g_free (privates->cif);
   g_callable_info_free_closure (privates->info, privates->closure);
   g_base_info_unref ((GIBaseInfo *) privates->info);
+
+  JSValueUnprotect (eng->context, object);
 }
 
 static void
@@ -42,15 +46,17 @@ seed_handle_closure (ffi_cif * cif, void *result, void **args, void *userdata)
   SeedNativeClosure *privates = userdata;
   gint num_args, i;
   JSValueRef *jsargs;
-  JSValueRef return_value;
-  JSValueRef exception = 0;
+  JSValueRef return_value, exception = 0;
   GITypeTag return_tag;
   GIArgInfo *arg_info;
   GITypeInfo *return_type;
-  GArgument rarg;
-  GArgument return_arg;
+  GITypeInfo *arg_type;
+  GITypeTag tag;
+  GArgument rarg, return_arg;
   JSContextRef ctx = JSGlobalContextCreateInGroup (context_group, 0);
-  
+  GArgument *arg = &rarg;
+  gchar *mes;
+
   seed_prepare_global_context (ctx);
 
   SEED_NOTE (INVOCATION, "Invoking closure of type: %s \n",
@@ -63,9 +69,6 @@ seed_handle_closure (ffi_cif * cif, void *result, void **args, void *userdata)
 
   for (i = 0; i < num_args; i++)
     {
-      GITypeInfo *arg_type;
-      GITypeTag tag;
-      GArgument *arg = &rarg;
 
       arg_info = g_callable_info_get_arg (privates->info, i);
       arg_type = g_arg_info_get_type (arg_info);
@@ -95,20 +98,28 @@ seed_handle_closure (ffi_cif * cif, void *result, void **args, void *userdata)
 	  arg->v_uint32 = *(guint32 *) args[i];
 	  break;
 	case GI_TYPE_TAG_LONG:
+	  arg->v_long = *(glong *) args[i];
+	  break;
 	case GI_TYPE_TAG_INT64:
-	  arg->v_int64 = *(glong *) args[i];
+	  arg->v_int64 = *(gint64 *) args[i];
 	  break;
 	case GI_TYPE_TAG_ULONG:
+	  arg->v_ulong = *(gulong *) args[i];
+	  break;
 	case GI_TYPE_TAG_UINT64:
-	  arg->v_uint64 = *(glong *) args[i];
+	  arg->v_uint64 = *(guint64 *) args[i];
 	  break;
 	case GI_TYPE_TAG_INT:
+	  arg->v_int = *(gint *) args[i];
+	  break;
 	case GI_TYPE_TAG_SSIZE:
+	  arg->v_ssize = *(gssize *) args[i];
+	  break;
 	case GI_TYPE_TAG_SIZE:
-	  arg->v_int32 = *(gint *) args[i];
+	  arg->v_size = *(gsize *) args[i];
 	  break;
 	case GI_TYPE_TAG_UINT:
-	  arg->v_uint32 = *(guint *) args[i];
+	  arg->v_uint = *(guint *) args[i];
 	  break;
 	case GI_TYPE_TAG_FLOAT:
 	  arg->v_float = *(gfloat *) args[i];
@@ -172,8 +183,7 @@ seed_handle_closure (ffi_cif * cif, void *result, void **args, void *userdata)
 
   if (exception)
     {
-      gchar *mes = seed_exception_to_string (ctx,
-					     exception);
+      mes = seed_exception_to_string (ctx, exception);
       g_warning ("Exception in closure marshal. %s \n", mes);
       g_free (mes);
       exception = 0;
@@ -197,7 +207,7 @@ seed_handle_closure (ffi_cif * cif, void *result, void **args, void *userdata)
       *(gint16 *) result = return_arg.v_int16;
       break;
     case GI_TYPE_TAG_UINT16:
-      return_arg.v_uint16 = *(guint16 *) args[i];
+      *(guint16 *) result = return_arg.v_uint16;
       break;
     case GI_TYPE_TAG_INT32:
       *(gint32 *) result = return_arg.v_int32;
@@ -206,20 +216,28 @@ seed_handle_closure (ffi_cif * cif, void *result, void **args, void *userdata)
       *(guint32 *) result = return_arg.v_uint32;
       break;
     case GI_TYPE_TAG_LONG:
+      *(glong *) result = return_arg.v_long;
+      break;
     case GI_TYPE_TAG_INT64:
-      *(glong *) result = return_arg.v_int64;
+      *(gint64 *) result = return_arg.v_int64;
       break;
     case GI_TYPE_TAG_ULONG:
+      *(gulong *) result = return_arg.v_ulong;
+      break;
     case GI_TYPE_TAG_UINT64:
-      *(glong *) result = return_arg.v_uint64;
+      *(guint64 *) result = return_arg.v_uint64;
       break;
     case GI_TYPE_TAG_INT:
+      *(gint *) result = return_arg.v_int;
+      break;
     case GI_TYPE_TAG_SSIZE:
+      *(gssize *) result = return_arg.v_ssize;
+      break;
     case GI_TYPE_TAG_SIZE:
-      *(gint *) result = return_arg.v_int32;
+      *(gsize *) result = return_arg.v_size;
       break;
     case GI_TYPE_TAG_UINT:
-      *(guint *) result = return_arg.v_uint32;
+      *(guint *) result = return_arg.v_uint;
       break;
     case GI_TYPE_TAG_FLOAT:
       *(gfloat *) result = return_arg.v_float;
@@ -271,7 +289,8 @@ seed_handle_closure (ffi_cif * cif, void *result, void **args, void *userdata)
 
 SeedNativeClosure *
 seed_make_native_closure (JSContextRef ctx,
-			  GICallableInfo * info, JSValueRef function)
+			  GICallableInfo * info,
+			  JSValueRef function)
 {
   ffi_cif *cif;
   ffi_closure *closure;
@@ -300,7 +319,9 @@ seed_make_native_closure (JSContextRef ctx,
   privates->function = function;
   privates->cif = cif;
 
-  closure = g_callable_info_prepare_closure (info, cif, seed_handle_closure, privates);
+  closure =
+    g_callable_info_prepare_closure (info, cif, seed_handle_closure,
+				     privates);
   privates->closure = closure;
 
   JSValueProtect (ctx, function);
@@ -316,27 +337,115 @@ seed_make_native_closure (JSContextRef ctx,
   return privates;
 }
 
-SeedClosure *
-seed_make_gclosure (JSContextRef ctx, JSObjectRef function, JSObjectRef this)
+static void
+closure_invalidated (gpointer data, GClosure * c)
+{
+  SeedClosure *closure = (SeedClosure *) c;
+
+  SEED_NOTE (FINALIZATION, "Finalizing closure.");
+  if (closure->user_data
+      && !JSValueIsUndefined (eng->context, closure->user_data))
+    JSValueUnprotect (eng->context, closure->user_data);
+  if (!JSValueIsUndefined (eng->context, closure->function))
+    JSValueUnprotect (eng->context, closure->function);
+
+  g_free (closure->description);
+
+}
+
+JSObjectRef
+seed_closure_get_callable (GClosure * c)
+{
+  return ((SeedClosure *) c)->function;
+}
+
+JSValueRef
+seed_closure_invoke (GClosure * closure, JSValueRef * args, guint argc,
+		     JSValueRef * exception)
+{
+  JSContextRef ctx = JSGlobalContextCreateInGroup (context_group, 0);
+  JSValueRef *real_args = g_newa (JSValueRef, argc + 1);
+  JSValueRef ret;
+  guint i;
+
+  seed_prepare_global_context (ctx);
+  for (i = 0; i < argc; i++)
+    real_args[i] = args[i];
+  args[argc] =
+    ((SeedClosure *) closure)->user_data ? ((SeedClosure *) closure)->
+    user_data : JSValueMakeNull (ctx);
+
+  ret =
+    JSObjectCallAsFunction (ctx, ((SeedClosure *) closure)->function, NULL,
+			    argc + 1, real_args, exception);
+  JSGlobalContextRelease ((JSGlobalContextRef) ctx);
+
+  return ret;
+}
+
+JSValueRef
+seed_closure_invoke_with_context (JSContextRef ctx, GClosure * closure,
+				  JSValueRef * args, guint argc,
+				  JSValueRef * exception)
+{
+  JSValueRef *real_args = g_newa (JSValueRef, argc + 1);
+  JSValueRef ret;
+  guint i;
+
+  for (i = 0; i < argc; i++)
+    real_args[i] = args[i];
+  args[argc] =
+    ((SeedClosure *) closure)->user_data ? ((SeedClosure *) closure)->
+    user_data : JSValueMakeNull (ctx);
+
+  ret =
+    JSObjectCallAsFunction (ctx, ((SeedClosure *) closure)->function, NULL,
+			    argc + 1, real_args, exception);
+
+  return ret;
+}
+
+GClosure *
+seed_closure_new (JSContextRef ctx, JSObjectRef function,
+		  JSObjectRef user_data, const gchar * description)
 {
   GClosure *closure;
 
   closure = g_closure_new_simple (sizeof (SeedClosure), 0);
+  g_closure_add_finalize_notifier (closure, 0, closure_invalidated);
   g_closure_set_marshal (closure, seed_signal_marshal_func);
 
-  ((SeedClosure *) closure)->function = function;
-  ((SeedClosure *) closure)->object = 0;
-  if (this && !JSValueIsNull (ctx, this))
-    {
-      JSValueProtect (ctx, this);
-      ((SeedClosure *) closure)->this = this;
-    }
-  else
-    ((SeedClosure *) closure)->this = 0;
-
   JSValueProtect (ctx, function);
+  ((SeedClosure *) closure)->function = function;
+  if (user_data && !JSValueIsNull (ctx, user_data))
+    {
+      ((SeedClosure *) closure)->user_data = user_data;
+      JSValueProtect (ctx, user_data);
+    }
 
-  return (SeedClosure *) closure;
+  if (description)
+    ((SeedClosure *) closure)->description = g_strdup (description);
+
+  return closure;
+}
+
+void
+seed_closure_warn_exception (GClosure * c,
+			     JSContextRef ctx, JSValueRef exception)
+{
+  JSObjectRef callable = seed_closure_get_callable (c);
+  gchar *name = seed_value_to_string (ctx,
+				      seed_object_get_property (ctx, callable,
+								"name"),
+				      NULL);
+  gchar *mes = seed_exception_to_string (ctx, exception);
+
+  g_warning ("Exception in closure (%p) for %s (handler %s). %s", c,
+	     ((SeedClosure *) c)->description,
+	     *name == '\0' ? "[anonymous]" : name, mes);
+
+  g_free (name);
+  g_free (mes);
 }
 
 JSClassDefinition seed_native_callback_def = {
